@@ -2,7 +2,8 @@ from __future__ import division
 import pymc as pm
 import numpy as np
 from scipy import stats
-from scipy.stats.mstats import mquantiles
+
+from kabuki.utils import stochastic_from_dist
 
 np.seterr(divide='ignore')
 
@@ -12,12 +13,10 @@ def wiener_like_contaminant(value, cont_x, v, sv, a, z, sz, t, st, t_min, t_max,
                             err, n_st, n_sz, use_adaptive, simps_err):
     """Log-likelihood for the simple DDM including contaminants"""
     return hddm.wfpt.wiener_like_contaminant(value, cont_x.astype(np.int32), v, sv, a, z, sz, t, st,
-                                                  t_min, t_max, err, n_st, n_sz, use_adaptive, simps_err)
+                                             t_min, t_max, err, n_st, n_sz, use_adaptive, simps_err)
 
-WienerContaminant = pm.stochastic_from_dist(name="Wiener Simple Diffusion Process",
-                                       logp=wiener_like_contaminant,
-                                       dtype=np.float,
-                                       mv=True)
+WienerContaminant = stochastic_from_dist(name="Wiener Simple Diffusion Process",
+                                         logp=wiener_like_contaminant)
 
 def general_WienerCont(err=1e-4, n_st=2, n_sz=2, use_adaptive=1, simps_err=1e-3):
     _like = lambda  value, cont_x, v, sv, a, z, sz, t, st, t_min, t_max, err=err, n_st=n_st, n_sz=n_sz, \
@@ -25,10 +24,8 @@ def general_WienerCont(err=1e-4, n_st=2, n_sz=2, use_adaptive=1, simps_err=1e-3)
     wiener_like_contaminant(value, cont_x, v, sv, a, z, sz, t, st, t_min, t_max,\
                             err=err, n_st=n_st, n_sz=n_sz, use_adaptive=use_adaptive, simps_err=simps_err)
     _like.__doc__ = wiener_like_contaminant.__doc__
-    return pm.stochastic_from_dist(name="Wiener Diffusion Contaminant Process",
-                                       logp=_like,
-                                       dtype=np.float,
-                                       mv=False)
+    return stochastic_from_dist(name="Wiener Diffusion Contaminant Process",
+                                logp=_like)
 
 def generate_wfpt_stochastic_class(wiener_params=None, sampling_method='cdf', cdf_range=(-5,5), sampling_dt=1e-4):
     """
@@ -45,21 +42,23 @@ def generate_wfpt_stochastic_class(wiener_params=None, sampling_method='cdf', cd
     #set wiener_params
     if wiener_params is None:
         wiener_params = {'err': 1e-4, 'n_st':2, 'n_sz':2,
-                      'use_adaptive':1,
-                      'simps_err':1e-3,
-                      'w_outlier': 0.1}
+                         'use_adaptive':1,
+                         'simps_err':1e-3,
+                         'w_outlier': 0.1}
     wp = wiener_params
 
     #create likelihood function
     def wfpt_like(x, v, sv, a, z, sz, t, st, p_outlier=0):
-        return hddm.wfpt.wiener_like(x, v, sv, a, z, sz, t, st, p_outlier=p_outlier, **wp)
+        return hddm.wfpt.wiener_like(x['rt'].values, v, sv, a, z, sz, t, st, p_outlier=p_outlier, **wp)
 
 
     #create random function
-    def random(v, sv, a, z, sz, t, st, p_outlier, size=None):
-        param_dict = {'v': v, 'z': z, 't': t, 'a': a, 'sz': sz, 'sv': sv, 'st': st}
-        return hddm.generate.gen_rts(param_dict, method=sampling_method,
-                                     samples=size, dt=sampling_dt, range_=cdf_range)
+    def random(self):
+        return hddm.utils.flip_errors(hddm.generate.gen_rts(method=sampling_method,
+                                                            size=self.shape, dt=sampling_dt,
+                                                            range_=cdf_range,
+                                                            structured=True,
+                                                            **self.parents.value))
 
 
     #create pdf function
@@ -72,12 +71,13 @@ def generate_wfpt_stochastic_class(wiener_params=None, sampling_method='cdf', cd
         return hddm.cdfdif.dmat_cdf_array(x, w_outlier=wp['w_outlier'], **self.parents)
 
     #create wfpt class
-    wfpt = pm.stochastic_from_dist('wfpt', wfpt_like, random=random)
+    wfpt = stochastic_from_dist('wfpt', wfpt_like)
 
     #add pdf and cdf_vec to the class
     wfpt.pdf = pdf
     wfpt.cdf_vec = lambda self: hddm.wfpt.gen_cdf_using_pdf(time=cdf_range[1], **dict(self.parents.items() + wp.items()))
     wfpt.cdf = cdf
+    wfpt.random = random
 
     #add quantiles functions
     add_quantiles_functions_to_pymc_class(wfpt)
@@ -92,13 +92,24 @@ def add_quantiles_functions_to_pymc_class(pymc_class):
     """
 
     #turn pymc node into the final wfpt_node
-    def compute_quantiles_stats(self, quantiles):
+    def compute_quantiles_stats(self, quantiles=(0.1, 0.3, 0.5, 0.7, 0.9)):
         """
         compute quantiles statistics
         Input:
             quantiles : sequence
                 the sequence of quantiles,  e.g. (0.1, 0.3, 0.5, 0.7, 0.9)
         """
+        try:
+            if all(self._quantiles_edges == np.asarray(quantiles)):
+                return
+        except AttributeError:
+            pass
+
+        if hasattr(self, '_is_average_node'):
+            raise AttributeError('cannot recompute stats of average model')
+
+        self._quantiles_edges = np.asarray(quantiles)
+
         data = self.value
 
         #get proportion of data fall between the quantiles
@@ -109,28 +120,37 @@ def add_quantiles_functions_to_pymc_class(pymc_class):
         self._n_samples = len(data)
 
         # extract empirical RT at the quantiles
-        ub_emp_rt = mquantiles(data[data>0], prob=quantiles)
-        lb_emp_rt = -mquantiles(-data[data<0], prob=quantiles)
+        self._empirical_quantiles = hddm.utils.data_quantiles(data, quantiles)
+        ub_emp_rt = self._empirical_quantiles[1]
+        lb_emp_rt = -self._empirical_quantiles[0]
         self._emp_rt = np.concatenate((lb_emp_rt[::-1], np.array([0.]), ub_emp_rt))
 
         #get frequency of observed values
         freq_obs = np.zeros(len(proportion))
-        freq_obs[:len(quantiles)+1] = sum(data<0) * neg_proportion
-        freq_obs[len(quantiles)+1:] = sum(data>0) * pos_proportion
+        freq_obs[:len(quantiles)+1] = sum(data.rt<0) * neg_proportion
+        freq_obs[len(quantiles)+1:] = sum(data.rt>0) * pos_proportion
         self._freq_obs = freq_obs
 
-    def set_quantiles_stats(self, n_samples, emp_rt, freq_obs):
+    def set_quantiles_stats(self, quantiles, n_samples, emp_rt, freq_obs, p_upper):
         """
         set quantiles statistics (used when one do not to compute the statistics from the stochastic's value)
         """
+        self._quantiles_edges = np.asarray(quantiles)
         self._n_samples = n_samples
         self._emp_rt = emp_rt
         self._freq_obs = freq_obs
 
-    def get_quantiles_stats(self):
+        nq = len(quantiles)
+        q_lower = -emp_rt[:nq][::-1]
+        q_upper = emp_rt[nq+1:]
+        self._empirical_quantiles = (q_lower, q_upper,p_upper)
+
+    def get_quantiles_stats(self, quantiles=(0.1, 0.3, 0.5, 0.7, 0.9)):
         """
-        get quantiles statistics (after they were computed using compute_quantiles_stats_
+        get quantiles statistics (after they were computed using compute_quantiles_stats)
         """
+        self.compute_quantiles_stats(quantiles)
+
         stats = {'n_samples': self._n_samples, 'emp_rt': self._emp_rt, 'freq_obs': self._freq_obs}
         return stats
 
@@ -175,9 +195,28 @@ def add_quantiles_functions_to_pymc_class(pymc_class):
             return -np.inf
         return 2 * sum(self._freq_obs * np.log(theo_proportion))
 
-    def quantiles(self, quantiles=(.1, .3, .5, .7, .9)):
-        quantiles = np.asarray(quantiles)
+    def empirical_quantiles(self, quantiles=(.1, .3, .5, .7, .9)):
+        """
+        return the quantiles of the Stochastic's value
+        Output:
+            q_lower - lower boundary quantiles
+            q_upper - upper_boundary_quantiles
+            p_upper - probability of hitting the upper boundary
+        """
+        self.compute_quantiles_stats(quantiles)
 
+        return self._empirical_quantiles
+
+    def theoretical_quantiles(self, quantiles=(.1, .3, .5, .7, .9)):
+        """
+        return the theoretical quantiles based on Stochastic's parents
+        Output:
+            q_lower - lower boundary quantiles
+            q_upper - upper_boundary_quantiles
+            p_upper - probability of hitting the upper boundary
+        """
+
+        quantiles = np.asarray(quantiles)
         # generate CDF
         x_lower, cdf_lower, x_upper, cdf_upper = hddm.wfpt.split_cdf(*self.cdf_vec())
 
@@ -185,13 +224,11 @@ def add_quantiles_functions_to_pymc_class(pymc_class):
         lower_idx = np.searchsorted(cdf_lower, quantiles*cdf_lower[-1])
         upper_idx = np.searchsorted(cdf_upper, quantiles*cdf_upper[-1])
 
-        q_vals_lower = x_lower[lower_idx]
-        q_vals_upper = x_upper[upper_idx]
+        q_lower = x_lower[lower_idx]
+        q_upper = x_upper[upper_idx]
+        p_upper = cdf_upper[-1]
 
-        q_lower = np.vstack((q_vals_lower, quantiles*cdf_lower[-1]))
-        q_upper = np.vstack((q_vals_upper, quantiles*cdf_upper[-1]))
-
-        return (q_lower, q_upper)
+        return (q_lower, q_upper,p_upper)
 
     pymc_class.compute_quantiles_stats = compute_quantiles_stats
     pymc_class.set_quantiles_stats = set_quantiles_stats
@@ -199,8 +236,8 @@ def add_quantiles_functions_to_pymc_class(pymc_class):
     pymc_class.chisquare = chisquare
     pymc_class.gsquare = gsquare
     pymc_class._get_theoretical_proportion = _get_theoretical_proportion
-    pymc_class.quantiles = quantiles
+    pymc_class.empirical_quantiles = empirical_quantiles
+    pymc_class.theoretical_quantiles = theoretical_quantiles
 
 #create default Wfpt class
 Wfpt = generate_wfpt_stochastic_class()
-
