@@ -4,7 +4,6 @@ from hddm_truncated import HDDMTruncated
 from hddm_transformed import HDDMTransformed
 from hddm_stimcoding import HDDMStimCoding
 from hddm_regression import HDDMRegressor
-from hlba_truncated import HLBA
 
 __all__ = ['AccumulatorModel',
            'HDDMBase',
@@ -12,6 +11,5 @@ __all__ = ['AccumulatorModel',
            'HDDMTruncated',
            'HDDMStimCoding',
            'HDDMRegressor',
-           'HLBA',
            'HDDMTransformed',
 ]
